@@ -17,15 +17,15 @@
 import inspect
 import logging
 import os
-import traceback
 
 from PyQt5 import QtCore
 from PyQt5.QtCore import pyqtSignal, QUrl
 from PyQt5.QtGui import QDesktopServices, QColor, QFont
-from PyQt5.QtWidgets import QFileSystemModel, QTreeView, QWidget, QVBoxLayout, QAbstractItemView, QFileDialog, QMenu, \
-	QAction, QListWidget, QListWidgetItem, QTreeWidget, QMessageBox, QTreeWidgetItem, QTextEdit
+from PyQt5.QtWidgets import QFileSystemModel, QTreeView, QWidget, QVBoxLayout, QAbstractItemView, QMenu, \
+	QAction, QListWidget, QListWidgetItem, QTreeWidget, QTreeWidgetItem, QTextEdit
 
-from pre_workbench import configs, guihelper
+from pre_workbench import configs
+from pre_workbench.configs import getIcon
 from pre_workbench.errorhandler import ConsoleWindowLogHandler
 from pre_workbench.genericwidgets import filledColorIcon
 from pre_workbench.guihelper import navigate
@@ -40,9 +40,10 @@ from pre_workbench.util import PerfTimer, truncate_str
 
 class FileBrowserWidget(QWidget):
 	on_open = pyqtSignal(str)
-	def __init__(self):
+	def __init__(self, rootFolder):
 		super().__init__()
 		self.initUI()
+		self.setRoot(rootFolder)
 
 	def initUI(self):
 		self.model = QFileSystemModel()
@@ -82,20 +83,14 @@ class FileBrowserWidget(QWidget):
 			else:
 				for wndTyp, meta in WindowTypes.types:
 					text = 'Open with '+meta.get('displayName', meta['name'])
-					print(wndTyp, meta)
-					ctx.addAction(QAction(text, self, statusTip=text,
+					ctx.addAction(QAction(text, self, statusTip=text, icon=getIcon(meta.get('icon', 'document.png')),
 											   triggered=lambda dummy, meta=meta: navigate("WINDOW", "Type="+meta['name'], "FileName="+selectedFile)))
 				ctx.addSeparator()
-
-		ctx.addAction("Set root folder ...", lambda: self.selectRootFolder(preselect=selectedFolder))
+		else:
+			ctx.addAction("Open in file manager", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(self.model.rootPath())))
+		#ctx.addAction("Set root folder ...", lambda: self.selectRootFolder(preselect=selectedFolder))
 		ctx.exec(self.tree.viewport().mapToGlobal(point))
 
-	def selectRootFolder(self, preselect=None):
-		if preselect == None: preselect = self.rootFolder
-		dir = QFileDialog.getExistingDirectory(self,"Set root folder", preselect)
-		if dir != None:
-			self.setRoot(dir)
-
 	def setRoot(self, dir):
 		self.rootFolder = dir
 		self.model.setRootPath(dir)
@@ -113,10 +108,6 @@ class FileBrowserWidget(QWidget):
 			return { "sel": info.absoluteFilePath(), "root": self.rootFolder, "hs": self.tree.header().saveState() }
 
 	def restoreState(self, state):
-		try:
-			self.setRoot(state["root"])
-		except:
-			self.setRoot(guihelper.CurrentProject.projectFolder)
 		try:
 			idx = self.model.index(state["sel"])
 			if idx.isValid():
@@ -153,6 +144,7 @@ class MdiWindowListWidget(QWidget):
 		logging.debug("MdiWindowListWidget.updateWindowList (len=%d)", len(wndList))
 		self.list.clear()
 		for window in wndList:
+			if not hasattr(window.widget(), 'child_wnd_meta'): continue
 			listitem = QListWidgetItem(window.windowTitle())
 			listitem.setData(QtCore.Qt.UserRole, window.widget().objectName())
 			listitem.setIcon(window.windowIcon())
@@ -164,9 +156,15 @@ class MdiWindowListWidget(QWidget):
 		if item is not None:
 			ctx.addAction("Close window", lambda: self.closeWindow(item))
 
-		ctx.addAction("New window", lambda: self.newWindow())
+		ctx.addAction("Close all windows", lambda: self.closeAllWindows())
 		ctx.exec(self.list.viewport().mapToGlobal(point))
 
+	def onSubWindowActivated(self, window):
+		if window is None: return
+		for i in range(self.list.count()):
+			if self.list.item(i).data(QtCore.Qt.UserRole) == window.objectName():
+				self.list.setCurrentRow(i)
+
 
 class StructInfoTreeWidget(QWidget):
 	def __init__(self):
@@ -181,17 +179,9 @@ class StructInfoTreeWidget(QWidget):
 		self.setLayout(windowLayout)
 
 	def show_grammar(self, fic):
+		if not self.isVisible(): return
 		self.tree.set(fic)
 
-	def showEvent(self, QShowEvent):
-		self.parentWidget().parentWidget().on_grammar_update.connect(self.show_grammar)
-
-	def hideEvent(self, QHideEvent):
-		try:
-			self.parentWidget().parentWidget().on_grammar_update.disconnect(self.show_grammar)
-		except:
-			pass #sometimes it is not connected, probably hideEvent is called multiple times
-
 
 class StructInfoCodeWidget(QWidget):
 	def __init__(self):
@@ -258,16 +248,8 @@ class DataInspectorWidget(QWidget):
 	def restoreState(self, data):
 		if "hs" in data: self.fiTreeWidget.header().restoreState(data["hs"])
 
-	def showEvent(self, QShowEvent):
-		self.parentWidget().parentWidget().on_selected_bytes_update.connect(self.on_select_bytes)
-
-	def hideEvent(self, QHideEvent):
-		try:
-			self.parentWidget().parentWidget().on_selected_bytes_update.disconnect(self.on_select_bytes)
-		except:
-			pass #sometimes it is not connected, probably hideEvent is called multiple times
-
 	def on_select_bytes(self, selbytes):#buffer:ByteBuffer, range:Range):
+		if not self.isVisible(): return
 		with PerfTimer("DataInspector parsing"):
 			parse_context = AnnotatingParseContext(self.fiTreeWidget.formatInfoContainer, selbytes) #buffer.getBytes(range.start, range.length()))
 			try:
@@ -314,18 +296,9 @@ class RangeListWidget(QWidget):
 		windowLayout.setContentsMargins(0,0,0,0)
 		self.setLayout(windowLayout)
 
-	def showEvent(self, QShowEvent):
-		self.parentWidget().parentWidget().on_meta_update.connect(self.on_select_hexview_range)
-
-	def hideEvent(self, QHideEvent):
-		try:
-			self.parentWidget().parentWidget().on_meta_update.disconnect(self.on_select_hexview_range)
-		except:
-			pass #sometimes it is not connected, probably hideEvent is called multiple times
-
-	def on_select_hexview_range(self, event_id, sender):
+	def on_meta_update(self, event_id, sender):
 		with PerfTimer("RangeListWidget update"):
-			if event_id != "hexview_range" or sender is None: return
+			if event_id != "hexview_range" or sender is None or not self.isVisible(): return
 			self.treeView.clear()
 			for d in sender.buffers[0].matchRanges(overlaps=sender.selRange()):
 				root = QTreeWidgetItem(self.treeView)
@@ -371,7 +344,6 @@ class SelectionHeuristicsConfigWidget(QWidget):
 			self.listView.addItem(item)
 
 	def currentItemChanged(self, current, previous):
-		print(current)
 		self.infoBox.setText(inspect.cleandoc(current.data(self.HELPER_ROLE).__doc__) if current is not None else "")
 
 	def itemChanged(self, item):
