@@ -72,6 +72,7 @@ class FileDropLineEdit(QLineEdit):
 			self.setText(e.mimeData().text())
 			e.accept()
 
+
 class ColorSelectLineEdit(QLineEdit):
 	def __init__(self, *__args):
 		super().__init__(*__args)
@@ -81,12 +82,12 @@ class ColorSelectLineEdit(QLineEdit):
 		self.addAction(self.colorSelectAction, QLineEdit.TrailingPosition)
 	def selectColor(self):
 		result = QColorDialog.getColor(QColor(self.text()))
-		print(result)
 		if result:
 			self.setText(result.name())
 	def onTextChanged(self, newText):
 		self.colorSelectAction.setIcon(filledColorIcon(newText, 16))
 
+
 class FontSelectLineEdit(QLineEdit):
 	def __init__(self, *__args):
 		super().__init__(*__args)
@@ -97,17 +98,16 @@ class FontSelectLineEdit(QLineEdit):
 	def selectFont(self):
 		initial = QFont(); initial.fromString(self.text())
 		result, ok = QFontDialog.getFont(initial, self)
-		print(result, result.toString())
 		if ok:
 			self.setText(result.toString())
 
 
-
 def filledColorIcon(color, size):
 	pix = QPixmap(size, size)
 	pix.fill(QColor(color))
 	return QIcon(pix)
 
+
 class SettingsGroup(QGroupBox):
 	item_changed = pyqtSignal(str, str)
 
@@ -276,19 +276,6 @@ class ExpandWidget(QWidget):
 			self.setSizePolicy(self.body.sizePolicy())
 
 
-
-def printsizepolicy(pol):
-	print("controlType", pol.controlType())
-	print("expandingDirections", pol.expandingDirections())
-	print("horizontalPolicy", pol.horizontalPolicy())
-	print("horizontalStretch", pol.horizontalStretch())
-	print("verticalPolicy", pol.verticalPolicy())
-	print("verticalStretch", pol.verticalStretch())
-
-
-
-
-
 class MdiFile:
 	sequenceNumber = 1
 	def initMdiFile(self, fileName=None, isUntitled=False, patterns="All Files (*.*)", defaultNamePattern="untitled%d.txt"):
