@@ -30,7 +30,7 @@ from PyQt5.QtWidgets import QMainWindow, QAction, QFileDialog, QWidget, QVBoxLay
 	QMdiArea, QDockWidget, QMessageBox, QToolButton, QMenu, QLabel, QApplication
 from PyQtAds import ads
 
-from pre_workbench.guihelper import NavigateCommands, GlobalEvents
+from pre_workbench.guihelper import NavigateCommands, GlobalEvents, navigateBrowser
 from pre_workbench import configs, SettingsSection, guihelper
 # noinspection PyUnresolvedReferences
 from pre_workbench import textfile
@@ -221,18 +221,20 @@ class WorkbenchMain(QMainWindow):
 			#fileMenu.addAction(newAct)
 			newTbMenu.addAction(newAct)
 		mainToolbar.addWidget(newTbAct)
-		fileMenu.addSeparator()
 		#fileMenu.addAction(self.newAct)
 
-		fileMenu.addAction(self.loadProjectAct)
-		recentProjectMenu = fileMenu.addMenu('&Recent projects')
-		for project in configs.getValue("ProjectMru", []):
-			recentProjectMenu.addAction(project, lambda dir=project: self.openProjectInNewWindow(dir))
 		fileMenu.addAction(self.openAct)
 		fileMenu.addAction(self.saveAct)
 		fileMenu.addAction(self.saveAsAct)
 		fileMenu.addAction(self.reloadFileAct)
 		fileMenu.addSeparator()
+
+		fileMenu.addAction(self.loadProjectAct)
+		recentProjectMenu = fileMenu.addMenu('&Recent projects')
+		for project in configs.getValue("ProjectMru", []):
+			recentProjectMenu.addAction(project, lambda dir=project: self.openProjectInNewWindow(dir))
+		fileMenu.addSeparator()
+
 		self.mruActions = []
 		for _ in range(MRU_MAX):
 			a = fileMenu.addAction("-")
@@ -281,8 +283,10 @@ class WorkbenchMain(QMainWindow):
 
 		##### HELP #####
 		helpMenu = menubar.addMenu("&Help")
-		helpMenu.addAction(QAction("Getting started", self))
-		helpMenu.addAction(QAction("About PRE Workbench", self, triggered=lambda: self.onAboutBox(),
+		helpMenu.addAction(QAction("Getting started", self, triggered=lambda: navigateBrowser("https://luelista.github.io/pre_workbench/getting-started")))
+		helpMenu.addAction(QAction("Syntax reference", self, triggered=lambda: navigateBrowser("https://luelista.github.io/pre_workbench/syntax-reference")))
+		helpMenu.addAction(QAction("Issue tracker", self, triggered=lambda: navigateBrowser("https://github.com/luelista/pre_workbench/issues")))
+		helpMenu.addAction(QAction("About PRE Workbench", self, triggered=lambda: self.showAboutBox(),
 								   menuRole=QAction.AboutRole))
 
 		mainToolbar.addAction(self.openAct)
@@ -311,22 +315,22 @@ class WorkbenchMain(QMainWindow):
 		self.dockWidgets = {}
 		self.createDockWnd("Project Files", "folder-tree.png", FileBrowserWidget(self.project.projectFolder), ads.LeftDockWidgetArea, showFirstRun=True)
 		self.dockWidgets["Project Files"].on_open.connect(self.openFile)
+		self.createDockWnd("Window List", "applications-stack.png", MdiWindowListWidget(), ads.LeftDockWidgetArea)
 		self.zoomWindow = DynamicDataWidget()
 		self.zoomWindow.on_meta_update.connect(self.onMetaUpdateRaw)
 		self.createDockWnd("Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea)
 		self.on_zoom_update.connect(lambda content: self.zoomWindow.setContents(content))
-		self.createDockWnd("Data Source Log", "bug.png", LogWidget("DataSource"), ads.BottomDockWidgetArea)
+		self.createDockWnd("Data Inspector", "user-detective-gray.png", DataInspectorWidget(), ads.BottomDockWidgetArea, showFirstRun=True)
+		self.on_selected_bytes_update.connect(self.dockWidgets["Data Inspector"].on_select_bytes)
+		self.createDockWnd("Data Source Log", "terminal--exclamation.png", LogWidget("DataSource"), ads.TopDockWidgetArea)
 
-		self.createDockWnd("Window List", "applications-stack.png", MdiWindowListWidget(), ads.LeftDockWidgetArea)
 		self.createDockWnd("Grammar Definition Tree", "tree.png", StructInfoTreeWidget())
 		self.on_grammar_update.connect(self.dockWidgets["Grammar Definition Tree"].show_grammar)
 		self.createDockWnd("Grammar Definition Code", "tree--pencil.png", StructInfoCodeWidget(), showFirstRun=True)
 		self.createDockWnd("Grammar Parse Result", "tree--arrow.png", RangeTreeDockWidget(), showFirstRun=True)
-		self.createDockWnd("Data Inspector", "user-detective-gray.png", DataInspectorWidget(), ads.BottomDockWidgetArea, showFirstRun=True)
-		self.on_selected_bytes_update.connect(self.dockWidgets["Data Inspector"].on_select_bytes)
 		self.createDockWnd("Selected Ranges", "bookmarks.png", RangeListWidget(), showFirstRun=True)
 		self.on_meta_update.connect(self.dockWidgets["Selected Ranges"].on_meta_update)
-		self.createDockWnd("Selection Heuristics", "clipboard-task.png", SelectionHeuristicsConfigWidget(), showFirstRun=True)
+		self.createDockWnd("Selection Heuristics", "clipboard-task.png", SelectionHeuristicsConfigWidget())
 
 		self.mdiArea.focusedDockWidgetChanged.connect(self.onSubWindowActivated)
 		self.mdiArea.focusedDockWidgetChanged.connect(self.dockWidgets["Window List"].onSubWindowActivated)
@@ -347,6 +351,12 @@ class WorkbenchMain(QMainWindow):
 		self.setWindowTitle(f'PRE Workbench - {self.project.projectFolder}')
 		self.show()
 
+	def showAboutBox(self):
+		QMessageBox.about(self, "PRE Workbench", "Protocol Reverse Engineering Workbench\n\n"
+												 "Copyright (c) 2022 Mira Weller\n"
+												 "Licensed under the GNU General Public License Version 3\n\n"
+												 "https://github.com/luelista/pre_workbench")
+
 	def onPreferences(self):
 		res = showPreferencesDlg(configs.configDefinitions, configs.configDict, "Preferences", self)
 		if res is not None:
